
#' compute the variogram
#'
#' @param path               `character` where the simulations are to be found
#' @param simulation_id      `character` the id of the simulation to read
#' @param response_var       `character` the variable whose response we are studying
#' @param factor_vars        `character` vector the factor names for which sensitivity will be estimated
#' @param response_reduce_fn `fn` the function to be used to reduce the response variable to one value (default=max)
#' @param factor_reduce_fn   `list(fn)` the functions to be used to reduce the factors variable to one value
#' @return the json string to embed this in the CAMP boxmodel interface
#' @export
#'
#' @import data.table
#' @import readcamp
#'
#' @example
#' test <-variogram( path = "/gpfs/scratch/bsc32/bsc032637//out/", simulation_id = "2027582", response_var = "O3", factor_vars = c("TOL","NO", "NO2","solar_zenith_angle"), response_reduce_fn=max, factor_reduce_fn = list(data.table::first, data.table::first, data.table::first, mean) )
variogram <- function(
  path,
  simulation_id,
  response_var,
  factor_vars,
  response_reduce_fn = max,
  factor_reduce_fn = rep(data.table::first, length(factor_vars))) {
    
  library(data.table)

  stopifnot(inherits(path, "character"), length(path) == 1)
  stopifnot(inherits(simulation_id, "character"), length(simulation_id) == 1)
  stopifnot(inherits(response_var, "character"), length(response_var) == 1)
  stopifnot(inherits(factor_vars, "character"))
  stopifnot(inherits(response_reduce_fn, "function"), length(response_reduce_fn) == 1)
  stopifnot(inherits(factor_reduce_fn, "list"), length(factor_reduce_fn) == length(factor_vars))
  
  for (fn in factor_reduce_fn) {
    stopifnot(inherits(fn, "function"))
  }

  simulation_path <- paste0(path, "/", simulation_id)
  stopifnot(dir.exists(simulation_path))

  gas_names <- readcamp::list_gas_names(simulation_path)

  gas_species_to_read <- c()
  other_vars_to_read <- c()

  if (response_var %in% gas_names) {
    gas_species_to_read <- c(gas_species_to_read, response_var)
  } else {
    other_vars_to_read <- c(other_vars_to_read, response_var)
  }

  for (factor in factor_vars) {
    if (factor %in% gas_names) {
      gas_species_to_read <- c(gas_species_to_read, factor)
    } else {
      other_vars_to_read <- c(other_vars_to_read, factor)
    }
  }

  print("the following vars will be read from the gas_concentration_array")
  print(gas_species_to_read)

  print("the following vars will be read directly from the netcdf")
  print(other_vars_to_read)

  ### READ THE GAS CONCENTRATIONS
  dataset <- readcamp::get_gas_species_concentrations(
    path = simulation_path,
    species = gas_species_to_read,
    max_to_take = 12)


  dataset <- data.table::dcast(
    dataset,
    cell + time ~ species,
    value.var = "gas_species_concentrations"
    )

  ### READ OTHER VARIABLES
  env_vars <- readcamp::get_var_from_all_cells(simulation_path, other_vars_to_read, max_to_take = 12)

  ### MERGE AND REDUCE  
  dataset <- dataset[env_vars, on = c("cell", "time")]

  reduced_dataset <- dataset[
    ,
    .(response = response_reduce_fn(get(response_var))),
    by = c("cell")
  ]
  data.table::setnames(reduced_dataset, "response", response_var)

  for (i in seq_along(factor_vars)) {
    factor <- factor_vars[i]
    reduce_fn <- factor_reduce_fn[[i]]

    temp <- dataset[
      ,
      .(temp_factor = reduce_fn(get(factor))),
      by = c("cell")
    ]

    data.table::setnames(temp, "temp_factor", factor)

    reduced_dataset <- reduced_dataset[
      temp,
      on = c("cell")
    ]
  }

  return(reduced_dataset)
}